/*
 * Copyright 2013-2014 eBay Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kylinolap.metadata;

import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.kylinolap.common.util.LocalFileMetadataTestCase;
import com.kylinolap.metadata.model.cube.CubeDesc;
import com.kylinolap.metadata.model.schema.TableDesc;

/**
 * Created with IntelliJ IDEA. User: lukhan Date: 9/24/13 Time: 2:38 PM To
 * change this template use File | Settings | File Templates.
 */
public class MetadataManagerTest extends LocalFileMetadataTestCase {

    @Before
    public void setUp() throws Exception {
        this.createTestMetadata();
    }

    @After
    public void after() throws Exception {
        this.cleanupTestMetadata();
    }

    @Test
    public void testListAllTables() throws Exception {
        List<TableDesc> tables = MetadataManager.getInstance(this.getTestConfig()).listAllTables();
        Assert.assertNotNull(tables);
        Assert.assertTrue(tables.size() > 0);
    }

    @Test
    public void testFindTableByName() throws Exception {
        TableDesc table = MetadataManager.getInstance(this.getTestConfig()).getTableDesc("TEST_CAL_DT");
        Assert.assertNotNull(table);
        Assert.assertEquals("TEST_CAL_DT", table.getName());
    }

    @Test
    public void testGetInstance() throws Exception {

        Assert.assertNotNull(MetadataManager.getInstance(this.getTestConfig()));
        Assert.assertNotNull(MetadataManager.getInstance(this.getTestConfig()).listAllTables());
        Assert.assertTrue(MetadataManager.getInstance(this.getTestConfig()).listAllTables().size() > 0);
    }

    @Test
    public void testGetCubeDesc() throws Exception {
        CubeDesc cubeDesc = MetadataManager.getInstance(this.getTestConfig()).getCubeDesc("test_kylin_cube_with_slr_desc");
        Assert.assertNotNull(cubeDesc);
    }

}
